@@ -5,6 +5,7 @@ pub mod simplify;
 pub mod simulate;
 
 use crate::collections::{Map, Set};
+use crate::nfa::AcceptingState;
 use crate::range_map::{Range, RangeMap};
 
 use std::convert::TryFrom;
@@ -32,13 +33,13 @@ impl StateIdx {
 #[derive(Debug)]
 pub struct State<T, A> {
     // Is this the initial state of a rule set? This is important as failure transitions in initial
-    // states consume the current character, but failure transitions in other states don't.
+    // states consume the current character, but failure transitions in other states don't. (#12)
     initial: bool,
     char_transitions: Map<char, T>,
     range_transitions: RangeMap<T>,
     any_transition: Option<T>,
     end_of_input_transition: Option<T>,
-    accepting: Option<A>,
+    accepting: Vec<AcceptingState<A>>,
     // Predecessors of the state, used to inline code for a state with one predecessor in the
     // predecessor's code
     predecessors: Set<StateIdx>,
@@ -52,7 +53,7 @@ impl<T, A> State<T, A> {
             range_transitions: Default::default(),
             any_transition: None,
             end_of_input_transition: None,
-            accepting: None,
+            accepting: vec![],
             predecessors: Default::default(),
         }
     }
@@ -81,12 +82,8 @@ impl<A> DFA<StateIdx, A> {
         StateIdx(0)
     }
 
-    pub fn make_state_accepting(&mut self, state: StateIdx, value: A) {
-        // Give first rule priority
-        let accepting = &mut self.states[state.0].accepting;
-        if accepting.is_none() {
-            *accepting = Some(value);
-        }
+    pub fn make_state_accepting(&mut self, state: StateIdx, accept: AcceptingState<A>) {
+        self.states[state.0].accepting.push(accept);
     }
 
     pub fn new_state(&mut self) -> StateIdx {
@@ -95,6 +92,11 @@ impl<A> DFA<StateIdx, A> {
         new_state_idx
     }
 
+    #[cfg(test)]
+    pub fn is_accepting_state(&self, state: StateIdx) -> bool {
+        !self.states[state.0].accepting.is_empty()
+    }
+
     pub fn add_char_transition(&mut self, state: StateIdx, char: char, next: StateIdx) {
         let old = self.states[state.0].char_transitions.insert(char, next);
         assert!(
@@ -109,18 +111,14 @@ impl<A> DFA<StateIdx, A> {
         self.states[next.0].predecessors.insert(state);
     }
 
-    pub fn add_range_transition(
-        &mut self,
-        state: StateIdx,
-        range_begin: u32,
-        range_end: u32,
-        next: StateIdx,
-    ) {
-        self.states[state.0]
-            .range_transitions
-            .insert(range_begin, range_end, next);
+    pub fn set_range_transitions(&mut self, state: StateIdx, range_map: RangeMap<StateIdx>) {
+        assert!(self.states[state.0].range_transitions.is_empty());
 
-        self.states[next.0].predecessors.insert(state);
+        for range in range_map.iter() {
+            self.states[range.value.0].predecessors.insert(state);
+        }
+
+        self.states[state.0].range_transitions = range_map;
     }
 
     pub fn set_any_transition(&mut self, state: StateIdx, next: StateIdx) {
@@ -183,7 +181,6 @@ impl<A> DFA<StateIdx, A> {
         } in other.states
         {
             let mut new_char_transitions: Map<char, StateIdx> = Default::default();
-            let mut new_range_transitions: RangeMap<StateIdx> = Default::default();
             let mut new_any_transition: Option<StateIdx> = None;
             let mut new_end_of_input_transition: Option<StateIdx> = None;
 
@@ -191,15 +188,8 @@ impl<A> DFA<StateIdx, A> {
                 new_char_transitions.insert(char, StateIdx(next.0 + n_current_states));
             }
 
-            for range in range_transitions.iter() {
-                let values = &range.values;
-                assert_eq!(values.len(), 1);
-                new_range_transitions.insert(
-                    range.start,
-                    range.end,
-                    StateIdx(values[0].0 + n_current_states),
-                );
-            }
+            let new_range_transitions =
+                range_transitions.map(|state_idx| StateIdx(state_idx.0 + n_current_states));
 
             if let Some(next) = any_transition {
                 new_any_transition = Some(StateIdx(next.0 + n_current_states));
@@ -250,7 +240,7 @@ impl<A> Display for DFA<StateIdx, A> {
                 predecessors: _,
             } = state;
 
-            if accepting.is_some() {
+            if !accepting.is_empty() {
                 if *initial {
                     write!(f, "{:>5}:", format!("i*{}", state_idx))?;
                 } else {
@@ -267,25 +257,28 @@ impl<A> Display for DFA<StateIdx, A> {
             let mut first = true;
 
             for (char, next) in char_transitions.iter() {
-                first = false;
-
-                writeln!(f, "{:?} -> {}", char, next)?;
-            }
-
-            for Range { start, end, values } in range_transitions.iter() {
                 if !first {
                     write!(f, "      ")?;
                 } else {
                     first = false;
                 }
 
-                assert_eq!(values.len(), 1);
+                writeln!(f, "{:?} -> {}", char, next)?;
+            }
+
+            for Range { start, end, value } in range_transitions.iter() {
+                if !first {
+                    write!(f, "      ")?;
+                } else {
+                    first = false;
+                }
+
                 writeln!(
                     f,
                     "{:?} - {:?} -> {}",
                     char::try_from(*start).unwrap(),
                     char::try_from(*end).unwrap(),
-                    values[0]
+                    value,
                 )?;
             }
 
