@@ -4,7 +4,9 @@ pub mod simulate;
 use crate::ast::{Regex, Var};
 use crate::collections::{Map, Set};
 use crate::display::HashSetDisplay;
+use crate::range_map::{Range, RangeMap};
 use crate::regex_to_nfa;
+use crate::right_ctx::RightCtxIdx;
 
 /// Non-deterministic finite automate, parameterized on values of accepting states.
 #[derive(Debug)]
@@ -19,11 +21,17 @@ pub struct StateIdx(usize);
 #[derive(Debug)]
 struct State<A> {
     char_transitions: Map<char, Set<StateIdx>>,
-    range_transitions: Map<(char, char), Set<StateIdx>>,
+    range_transitions: RangeMap<Set<StateIdx>>,
     empty_transitions: Set<StateIdx>,
     any_transitions: Set<StateIdx>,
     end_of_input_transitions: Set<StateIdx>,
-    accepting: Option<A>,
+    accepting: Option<AcceptingState<A>>,
+}
+
+#[derive(Debug, Clone, Copy)]
+pub struct AcceptingState<A> {
+    pub value: A,
+    pub right_ctx: Option<RightCtxIdx>,
 }
 
 impl<A> State<A> {
@@ -50,7 +58,7 @@ impl<A> NFA<A> {
         StateIdx(0)
     }
 
-    pub fn get_accepting_state(&self, state: StateIdx) -> Option<&A> {
+    pub fn get_accepting_state(&self, state: StateIdx) -> Option<&AcceptingState<A>> {
         self.states[state.0].accepting.as_ref()
     }
 
@@ -64,7 +72,7 @@ impl<A> NFA<A> {
     pub fn range_transitions(
         &self,
         state: StateIdx,
-    ) -> impl Iterator<Item = (&(char, char), &Set<StateIdx>)> {
+    ) -> impl Iterator<Item = &Range<Set<StateIdx>>> {
         self.states[state.0].range_transitions.iter()
     }
 
@@ -85,16 +93,23 @@ impl<A> NFA<A> {
         new_state_idx
     }
 
-    pub fn add_regex(&mut self, bindings: &Map<Var, Regex>, re: &Regex, value: A) {
+    pub fn add_regex(
+        &mut self,
+        bindings: &Map<Var, Regex>,
+        re: &Regex,
+        right_ctx: Option<RightCtxIdx>,
+        value: A,
+    ) {
         let re_accepting_state = self.new_state();
-        self.make_state_accepting(re_accepting_state, value);
+
+        self.make_state_accepting(re_accepting_state, value, right_ctx);
 
         let re_initial_state = self.new_state();
         let nfa_initial_state = self.initial_state();
 
-        regex_to_nfa::add_re(self, bindings, re, re_initial_state, re_accepting_state);
-
         self.add_empty_transition(nfa_initial_state, re_initial_state);
+
+        regex_to_nfa::add_re(self, bindings, re, re_initial_state, re_accepting_state);
     }
 
     pub fn add_char_transition(&mut self, state: StateIdx, char: char, next: StateIdx) {
@@ -110,17 +125,31 @@ impl<A> NFA<A> {
     pub fn add_range_transition(
         &mut self,
         state: StateIdx,
-        range_begin: char,
+        range_start: char,
         range_end: char,
         next: StateIdx,
     ) {
-        let not_exists = self.states[state.0]
-            .range_transitions
-            .entry((range_begin, range_end))
-            .or_default()
-            .insert(next);
+        let mut set: Set<StateIdx> = Default::default();
+        set.insert(next);
+        self.states[state.0].range_transitions.insert(
+            range_start as u32,
+            range_end as u32,
+            set,
+            |values_1, values_2| values_1.extend(values_2.into_iter()),
+        );
+    }
 
-        assert!(not_exists, "add_range_transition");
+    pub fn add_range_transitions(&mut self, state: StateIdx, ranges: RangeMap<()>, next: StateIdx) {
+        let mut set: Set<StateIdx> = Default::default();
+        set.insert(next);
+
+        let ranges = ranges.map(|()| set.clone());
+
+        self.states[state.0]
+            .range_transitions
+            .insert_ranges(ranges.into_iter(), |values_1, values_2| {
+                values_1.extend(values_2.into_iter())
+            });
     }
 
     pub fn add_empty_transition(&mut self, state: StateIdx, next: StateIdx) {
@@ -141,9 +170,12 @@ impl<A> NFA<A> {
         assert!(not_exists, "add_end_of_input_transition");
     }
 
-    fn make_state_accepting(&mut self, state: StateIdx, value: A) {
-        // TODO: Avoid overriding here?
-        self.states[state.0].accepting = Some(value);
+    fn make_state_accepting(&mut self, state: StateIdx, value: A, right_ctx: Option<RightCtxIdx>) {
+        let old = self.states[state.0]
+            .accepting
+            .replace(AcceptingState { value, right_ctx });
+
+        assert!(old.is_none(), "make_state_accepting");
     }
 
     pub fn compute_state_closure(&self, states: &Set<StateIdx>) -> Set<StateIdx> {
@@ -187,10 +219,22 @@ impl<A> Display for NFA<A> {
                 accepting,
             } = state;
 
-            if accepting.is_some() {
-                write!(f, "{:>4}", format!("*{}", state_idx))?;
-            } else {
-                write!(f, "{:>4}:", state_idx)?;
+            match accepting {
+                Some(AcceptingState {
+                    value: _,
+                    right_ctx,
+                }) => match right_ctx {
+                    Some(right_ctx_idx) => {
+                        write!(f, "{:>4}", format!("*{}", state_idx),)?;
+                        write!(f, " (ctx {})", right_ctx_idx.as_usize())?;
+                    }
+                    None => {
+                        write!(f, "{:>4}", format!("*{}", state_idx))?;
+                    }
+                },
+                None => {
+                    write!(f, "{:>4}:", state_idx)?;
+                }
             }
 
             let mut first = true;
@@ -215,7 +259,7 @@ impl<A> Display for NFA<A> {
                 writeln!(f, "{:?} -> {}", char, HashSetDisplay(next))?;
             }
 
-            for ((range_begin, range_end), next) in range_transitions.iter() {
+            for range in range_transitions.iter() {
                 if !first {
                     write!(f, "     ")?;
                 } else {
@@ -225,9 +269,9 @@ impl<A> Display for NFA<A> {
                 writeln!(
                     f,
                     "{:?} - {:?} -> {}",
-                    range_begin,
-                    range_end,
-                    HashSetDisplay(next)
+                    range.start,
+                    range.end,
+                    HashSetDisplay(&range.value)
                 )?;
             }
 
