@@ -1,7 +1,7 @@
 use crate::collections::{Map, Set};
 use crate::dfa::DFA;
 use crate::nfa::NFA;
-use crate::range_map::RangeMap;
+use crate::range_map::{Range, RangeMap};
 
 use crate::dfa::StateIdx as DfaStateIdx;
 use crate::nfa::StateIdx as NfaStateIdx;
@@ -47,7 +47,7 @@ pub fn nfa_to_dfa<A: Clone>(nfa: &NFA<A>) -> DFA<DfaStateIdx, A> {
         finished_dfa_states.insert(current_dfa_state);
 
         let mut char_transitions: Map<char, Set<NfaStateIdx>> = Default::default();
-        let mut range_transitions: RangeMap<NfaStateIdx> = Default::default();
+        let mut range_transitions: RangeMap<Set<NfaStateIdx>> = Default::default();
         let mut any_transitions: Set<NfaStateIdx> = Default::default();
         let mut end_of_input_transitions: Set<NfaStateIdx> = Default::default();
 
@@ -65,12 +65,12 @@ pub fn nfa_to_dfa<A: Clone>(nfa: &NFA<A>) -> DFA<DfaStateIdx, A> {
             }
 
             // Collect range transitions
-            for ((range_begin, range_end), next_states) in nfa.range_transitions(nfa_state) {
-                let next_states: Vec<NfaStateIdx> = next_states.iter().copied().collect();
-                range_transitions.insert_values(
-                    *range_begin as u32,
-                    *range_end as u32,
-                    &next_states,
+            for range in nfa.range_transitions(nfa_state) {
+                range_transitions.insert(
+                    range.start,
+                    range.end,
+                    range.value.clone(),
+                    |states_1, states_2| states_1.extend(states_2.into_iter()),
                 );
             }
 
@@ -87,7 +87,7 @@ pub fn nfa_to_dfa<A: Clone>(nfa: &NFA<A>) -> DFA<DfaStateIdx, A> {
             // transition
             for range in range_transitions.iter() {
                 if range.contains(char) {
-                    for range_state in &range.values {
+                    for range_state in &range.value {
                         char_states.insert(*range_state);
                     }
                 }
@@ -108,8 +108,11 @@ pub fn nfa_to_dfa<A: Clone>(nfa: &NFA<A>) -> DFA<DfaStateIdx, A> {
             work_list.push(closure);
         }
 
+        let mut dfa_range_transitions: Vec<Range<DfaStateIdx>> =
+            Vec::with_capacity(range_transitions.len());
+
         for range in range_transitions.into_iter() {
-            let mut range_states: Set<NfaStateIdx> = range.values.into_iter().collect();
+            let mut range_states: Set<NfaStateIdx> = range.value;
 
             for any_next in &any_transitions {
                 range_states.insert(*any_next);
@@ -119,12 +122,23 @@ pub fn nfa_to_dfa<A: Clone>(nfa: &NFA<A>) -> DFA<DfaStateIdx, A> {
                 .compute_state_closure(&range_states)
                 .into_iter()
                 .collect();
+
             let dfa_state = dfa_state_of_nfa_states(&mut dfa, &mut state_map, closure.clone());
-            dfa.add_range_transition(current_dfa_state, range.start, range.end, dfa_state);
+
+            dfa_range_transitions.push(Range {
+                start: range.start,
+                end: range.end,
+                value: dfa_state,
+            });
 
             work_list.push(closure);
         }
 
+        dfa.set_range_transitions(
+            current_dfa_state,
+            RangeMap::from_non_overlapping_sorted_ranges(dfa_range_transitions),
+        );
+
         {
             let closure: BTreeSet<NfaStateIdx> = nfa
                 .compute_state_closure(&any_transitions)
